package sun.security.jca;

import java.util.Iterator;
import java.util.List;
import java.security.NoSuchProviderException;
import java.security.NoSuchAlgorithmException;
import java.security.Provider;

public class GetInstance
{
    private GetInstance() {
    }
    
    public static Provider.Service getService(String str, String str2) throws NoSuchAlgorithmException {
        Provider.Service service = Providers.getProviderList().getService(str, str2);
        if (service == null) {
            throw new NoSuchAlgorithmException(str2 + " " + str + " not available");
        }
        return service;
    }
    
    public static Provider.Service getService(String type, String s, String s2) throws NoSuchAlgorithmException, NoSuchProviderException {
        if (s2 == null || s2.length() == 0) {
            throw new IllegalArgumentException("missing provider");
        }
        Provider provider = Providers.getProviderList().getProvider(s2);
        if (provider == null) {
            throw new NoSuchProviderException("no such provider: " + s2);
        }
        Provider.Service service = provider.getService(type, s);
        if (service == null) {
            throw new NoSuchAlgorithmException("no such algorithm: " + s + " for provider " + s2);
        }
        return service;
    }
    
    public static Provider.Service getService(String type, String s, Provider provider) throws NoSuchAlgorithmException {
        if (provider == null) {
            throw new IllegalArgumentException("missing provider");
        }
        Provider.Service service = provider.getService(type, s);
        if (service == null) {
            throw new NoSuchAlgorithmException("no such algorithm: " + s + " for provider " + provider.getName());
        }
        return service;
    }
    
    public static List<Provider.Service> getServices(String s, String s2) {
        return Providers.getProviderList().getServices(s, s2);
    }
    
    @Deprecated
    public static List<Provider.Service> getServices(String s, List<String> list) {
        return Providers.getProviderList().getServices(s, list);
    }
    
    public static List<Provider.Service> getServices(List<ServiceId> list) {
        return Providers.getProviderList().getServices(list);
    }
    
    public static GetInstance.Instance getInstance(String str, Class<?> clazz, String str2) throws NoSuchAlgorithmException {
        ProviderList providerList = Providers.getProviderList();
        Provider.Service service = providerList.getService(str, str2);
        if (service == null) {
            throw new NoSuchAlgorithmException(str2 + " " + str + " not available");
        }
        try {
            return getInstance(service, clazz);
        }
        catch (NoSuchAlgorithmException ex) {
            for (Provider.Service service2 : providerList.getServices(str, str2)) {
                if (service2 == service) {
                    continue;
                }
                try {
                    return getInstance(service2, clazz);
                }
                catch (NoSuchAlgorithmException ex2) {
                    ex = ex2;
                }
            }
            throw ex;
        }
    }
    
    public static GetInstance.Instance getInstance(String str, Class<?> clazz, String str2, Object o) throws NoSuchAlgorithmException {
        List<Provider.Service> services = getServices(str, str2);
        NoSuchAlgorithmException ex = null;
        for (Provider.Service service : services) {
            try {
                return getInstance(service, clazz, o);
            }
            catch (NoSuchAlgorithmException ex2) {
                ex = ex2;
            }
        }
        if (ex != null) {
            throw ex;
        }
        throw new NoSuchAlgorithmException(str2 + " " + str + " not available");
    }
    
    public static GetInstance.Instance getInstance(String s, Class<?> clazz, String s2, String s3) throws NoSuchAlgorithmException, NoSuchProviderException {
        return getInstance(getService(s, s2, s3), clazz);
    }
    
    public static GetInstance.Instance getInstance(String s, Class<?> clazz, String s2, Object o, String s3) throws NoSuchAlgorithmException, NoSuchProviderException {
        return getInstance(getService(s, s2, s3), clazz, o);
    }
    
    public static GetInstance.Instance getInstance(String s, Class<?> clazz, String s2, Provider provider) throws NoSuchAlgorithmException {
        return getInstance(getService(s, s2, provider), clazz);
    }
    
    public static GetInstance.Instance getInstance(String s, Class<?> clazz, String s2, Object o, Provider provider) throws NoSuchAlgorithmException {
        return getInstance(getService(s, s2, provider), clazz, o);
    }
    
    public static GetInstance.Instance getInstance(Provider.Service service, Class<?> clazz) throws NoSuchAlgorithmException {
        Object instance = service.newInstance(null);
        checkSuperClass(service, instance.getClass(), clazz);
        return new GetInstance.Instance(service.getProvider(), instance);
    }
    
    public static GetInstance.Instance getInstance(Provider.Service service, Class<?> clazz, Object constructorParameter) throws NoSuchAlgorithmException {
        Object instance = service.newInstance(constructorParameter);
        checkSuperClass(service, instance.getClass(), clazz);
        return new GetInstance.Instance(service.getProvider(), instance);
    }
    
    public static void checkSuperClass(Provider.Service service, Class<?> clazz, Class<?> clazz2) throws NoSuchAlgorithmException {
        if (clazz2 == null) {
            return;
        }
        if (!clazz2.isAssignableFrom(clazz)) {
            throw new NoSuchAlgorithmException("class configured for " + service.getType() + ": " + service.getClassName() + " not a " + service.getType());
        }
        return;
    }
    
    public static final class Instance
    {
        public final Provider provider;
        public final Object impl;
        
        private Instance(Provider provider, Object impl) {
            this.provider = provider;
            this.impl = impl;
        }
        
        public Object[] toArray() {
            return new Object[] { this.impl, this.provider };
        }
    }
}
